@@ -696,7 +696,7 @@ class GA:
 
         return best_solution, best_solution_fitness, best_match_idx
 
-    def plot_result(self):
+    def plot_result(self, title="PyGAD - Iteration vs. Fitness", xlabel="Generation", ylabel="Fitness"):
 
         """
         Creates and shows a plot that summarizes how the fitness value evolved by generation. Can only be called after completing at least 1 generation.
@@ -711,9 +711,9 @@ class GA:
 
         matplotlib.pyplot.figure()
         matplotlib.pyplot.plot(self.best_solutions_fitness)
-        matplotlib.pyplot.title("PyGAD - Iteration vs. Fitness")
-        matplotlib.pyplot.xlabel("Iteration")
-        matplotlib.pyplot.ylabel("Fitness")
+        matplotlib.pyplot.title(title)
+        matplotlib.pyplot.xlabel(xlabel)
+        matplotlib.pyplot.ylabel(ylabel)
         matplotlib.pyplot.show()
 
     def save(self, filename):
